@@ -25,31 +25,25 @@
 #  Based on libwget.py (2005-10-29)
 #
 
+import collections
+import email
 import logging
-logger = logging.getLogger(__name__)
-
-from gpodder import util
-from gpodder import youtube
-from gpodder import vimeo
-from gpodder import escapist_videos
-import gpodder
-
+import mimetypes
+import os
+import os.path
+import shutil
 import socket
 import threading
-import shutil
-import os.path
-import os
 import time
-import collections
-
-import mimetypes
-import email
 import urllib.error
 import urllib.parse
 import urllib.request
-
 from email.header import decode_header
 
+import gpodder
+from gpodder import escapist_videos, util, vimeo, youtube
+
+logger = logging.getLogger(__name__)
 
 _ = gpodder.gettext
 
@@ -204,7 +198,22 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
     def __init__(self, channel):
         self.channel = channel
         self._auth_retry_counter = 0
-        urllib.request.FancyURLopener.__init__(self, None)
+        super().__init__()
+
+    def http_error(self, url, fp, errcode, errmsg, headers, data=None):
+        """Handle http errors.
+        Overriden to give retry=True to http_error_40{1,7}.
+        See https://github.com/python/cpython/commit/80f1b059714aeb1c6fc9f6ce1173bc8a51af7dd9
+        See python issue https://bugs.python.org/issue1368368
+        """
+        result = False
+        if errcode == 401:
+            result = self.http_error_401(url, fp, errcode, errmsg, headers, data=data, retry=True)
+        elif errcode == 407:
+            result = self.http_error_407(url, fp, errcode, errmsg, headers, data=data, retry=True)
+        if result:
+            return result
+        return super().http_error(url, fp, errcode, errmsg, headers, data=data)
 
     def http_error_default(self, url, fp, errcode, errmsg, headers):
         """
@@ -303,7 +312,7 @@ class DownloadURLOpener(urllib.request.FancyURLopener):
                 block = fp.read(bs)
             else:
                 block = fp.read(min(size - read, bs))
-            if block == "":
+            if len(block) == 0:
                 break
             read += len(block)
             tfp.write(block)
@@ -740,6 +749,7 @@ class DownloadTask(object):
 
             url = util.iri_to_url(url)
 
+            logger.info("Downloading %s", url)
             downloader = DownloadURLOpener(self.__episode.channel)
 
             # HTTP Status codes for which we retry the download
